@@ -10,6 +10,7 @@
 #include <R_ext/Utils.h>
 #include <R_ext/Random.h>
 #include <R.h>
+#include <Rinternals.h>
 
 #define ME(matrix,row,col) (((matrix)->entries)[(col) * ((matrix)->nr) + (row)])
 #define VE(vector,index) (((vector)->entries)[(index)])
@@ -52,36 +53,50 @@ int length_vector(vector *v);
 void print_a_matrix(matrix *M);
 
 extern void F77_SUB(dpotri)(const char* uplo, const int* n,
-		 double* a, const int* lda, int* info);
+			    double* a, const int* lda, int* info);
 
 extern void F77_SUB(dpotrf)(const char* uplo, const int* n,
-		 double* a, const int* lda, int* info);
+			    double* a, const int* lda, int* info);
 
 extern void F77_SUB(dgemm)(const char *transa, const char *transb, const int *m, \
-		const int *n, const int *k, const double *alpha,\
-		const double *a, const int *lda,\
-		const double *b, const int *ldb,\
-		const double *beta, double *c, const int *ldc);
+			   const int *n, const int *k, const double *alpha,\
+			   const double *a, const int *lda,\
+			   const double *b, const int *ldb,\
+			   const double *beta, double *c, const int *ldc);
 
 extern void F77_SUB(dgemv)(const char *trans, const int *m, const int *n,
-		const double *alpha, const double *a, const int *lda,
-		const double *x, const int *incx, const double *beta,
-		double *y, const int *incy);
+			   const double *alpha, const double *a, const int *lda,
+			   const double *x, const int *incx, const double *beta,
+			   double *y, const int *incy);
 
 extern void F77_SUB(dgetrf)(const int* m, const int* n, double* a, const int* lda,
-                 int* ipiv, int* info);
+			    int* ipiv, int* info);
 
 extern void F77_SUB(dgetri)(const int* n, double* a, const int* lda,
                  int* ipiv, double* work, const int* lwork, int* info);
 
 extern void F77_SUB(dqrdc2)(double *x, int *ldx, int *n, int *p,
-                      double *tol, int *rank,
-                      double *qraux, int *pivot, double *work);
+			    double *tol, int *rank,
+			    double *qraux, int *pivot, double *work);
 
 extern void F77_SUB(dtrco)(double*, int*, int*, double*, double*, int*);
 
+extern void F77_SUB(dgecon)(const char* norm, const int* n,
+			    const double* a, const int* lda,
+			    const double* anorm, double* rcond,
+			    double* work, int* iwork, int* info);
+
+extern double F77_NAME(dlange)(const char* norm, const int* m, const int* n,
+			       const double* a, const int* lda, double* work);
+
+
 void MtM(matrix *M, matrix *A);
 
+
+void cumsumM(matrix *M, matrix *Mout,int rev,int weighted,double *weights); 
+void cumsumM1pM2(matrix *M1, matrix *M2,matrix *At[],int rev,int weighted,double *weights,int nindex, int *index);
+void cumsumMpM(matrix *M1,matrix *At[],int rev,int weighted,double *weights, int nindex,int *index);
+
 void invertSPD(matrix *A, matrix *AI);
 void invertSPDunsafe(matrix *A, matrix *AI);
 
